///Wifi config 
#include <Arduino.h>
#include <ArduinoJson.h>
#include <ESP8266WiFi.h>
#include <ESP8266WiFiMulti.h>
#include <ESP8266HTTPClient.h>
#include <WiFiClient.h>
///Light config
#include <Wire.h>
#include <BH1750.h>
///DHT config
#include "DHT.h"
///config
#include "config.h"

void setup() {
  if (CONFIG_DEBUG) {
    Serial.begin(115200);
  }
  wifiSetup();
  lightSetup();
  dhtSetup();
  soilSetup();
}

void wifiSetup(){
  WiFi.begin(ssid, password);
  Serial.println("Connecting");
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected to WiFi network with IP Address: ");
  Serial.println(WiFi.localIP());

  Serial.println("Timer set to 5 seconds (timerDelay variable), it will take 5 seconds before publishing the first reading.");
}

void lightSetup(){
  
  // Inicializace I2C
  Wire.begin();
  // On esp8266 you can select SCL and SDA pins using Wire.begin(D4, D3);

  BH1750Check = lightMeter.begin();
  if (BH1750Check) {
    Serial.println(F("BH1750 Test begin"));
  }
  else if (!CONFIG_DEBUG){
    Serial.println(F("BH1750 Initialization FAILED"));
    //Wait and try setup light again
    delay(60000);
    lightSetup();
  } else {
        Serial.println(F("BH1750 Initialization FAILED"));
  }
}

void dhtSetup(){
  dht.begin();
}

void soilSetup(){
  pinMode(soilPin, INPUT);
}
  
void loop() {
  if ((millis() - lastTime) > timerDelay) {
    //Check WiFi connection status
    if (WiFi.status() == WL_CONNECTED) {
      WiFiClient client;
      HTTPClient http;

      // Your Domain name with URL path or IP address with path
      http.begin(client, serverName);
      http.addHeader("Content-Type", "application/json");

      String requestJsonBody;
      requestJsonBody = createDataObject();
      Serial.println(requestJsonBody);
      int httpResponseCode = http.POST(requestJsonBody);

      Serial.print("HTTP Response code: ");
      Serial.println(httpResponseCode);

      // Free resources
      http.end();
    }
    else {
      Serial.println("WiFi Disconnected");
    }
    lastTime = millis();
  }
}

String createDataObject(){
    // If you need an HTTP request with a content type: application/json, use the following:
    StaticJsonDocument<300> doc;

    // Add values in the document
    doc["apiKey"] = apiKey;
    doc["deviceID"] = deviceID;
    doc["temperature"] = getTemperature();
    doc["humidity"] = getHumidity();
    doc["soilHumidity"] = getSoilHumidity();
    doc["co2"] = getCO2();
    doc["light"] = getLight();

    // Serialize json data
    String json;
    serializeJson(doc, json);
    return json;
}

float getLight(){
  if (BH1750Check) {
    float lux = lightMeter.readLightLevel();
    return lux;
  }
}

float getTemperature(){
  float temp = dht.readTemperature();
  if (isnan(temp)) {
    Serial.println("Failed to read from DHT sensor!");
    return 1111;
  }else {
    return temp;
  }
}

float getHumidity(){
  float hum = dht.readHumidity();
  if (isnan(hum)) {
    Serial.println("Failed to read from DHT sensor!");
    return 1111;
  }else {
    return hum;
  }
}

float getCO2(){
  float co = analogRead(MQ2pin);
  return co;
}

bool getSoilHumidity(){
  bool soil = digitalRead(soilPin);
  return soil;
}
